MODULE Matrix;
IMPORT IO, Thread;

(*-------------------------------------------------------------------*)
(* Prozedur Init, uebernimmt Matrix: a, keine Rueckgabewerte, erzeugt*)
(* symmetrische Matrix, Elemente erhalten jew. Summe aus Zeilen- und *)
(* Spaltenindex.                                                     *)
(*-------------------------------------------------------------------*)

PROCEDURE Init(VAR a : T) =

BEGIN

 FOR i:=1 TO n DO

  FOR j:=1 TO n DO

   a[i,j]:= i+j;

  END;

 END;

END Init;

(*-------------------------------------------------------------------*)
(* Prozedur Put, liest Matrix: a, keine Rueckgabewerte, gibt Elemente*)
(* durch Tabulatoren getrennt aus.                                   *)
(*-------------------------------------------------------------------*)

PROCEDURE Put(READONLY a : T) =

BEGIN

 FOR i:=1 TO n DO

  FOR j:=1 TO n DO

   IO.PutInt(a[i,j]);
   IO.Put("\t");

  END;

   IO.Put("\n");

 END;
END Put;

(*-------------------------------------------------------------------*)
(* Prozedur Add, liest Matritzen: a,b als Werte, uebernimmt Matrix c,*)
(* keine Rueckgabewerte, formt um: c in Summe von a und b.           *)
(*-------------------------------------------------------------------*)

PROCEDURE Add(READONLY a,b : T; VAR c : T) =

BEGIN

 FOR i:=1 TO n DO

  FOR j:=1 TO n DO

   c[i,j]:= a[i,j]+b[i,j];

  END;

 END;
END Add;

(*-------------------------------------------------------------------*)
(* Prozedur ScalarProd, liest Matrix: a als Wert, liest Argument: b, *)
(* uebernimmt Matrix c, keine Rueckgabewerte, formt um: c in Produkt *)
(* der Elemente aus a multipliziert b                                *)
(*-------------------------------------------------------------------*)

PROCEDURE ScalarProd(READONLY a: T;READONLY b : INTEGER; VAR c : T) =

BEGIN

 FOR i:=1 TO n DO

  FOR j:=1 TO n DO

   c[i,j]:= a[i,j]*b;

  END;

 END;

END ScalarProd;

(*-------------------------------------------------------------------*)
(* Prozedur MultPar, liest Matrizen x,y, liefert MatProd in c        *)
(* zurueck. Unterteilt dazu in 4 Untermatrizen und ruft damit Mult   *)
(* in 3 zusaetzlichen Kontrollfaeden auf.                            *)
(*-------------------------------------------------------------------*)

PROCEDURE MultPar(READONLY x,y : T; VAR c : T) =

VAR teins, tzwei, tdrei : Thread.T;
VAR ceins := NEW(Closure);
VAR czwei := NEW(Closure);
VAR cdrei := NEW(Closure);
VAR cvier := NEW(Closure);

BEGIN

(*X,Y [1,1]*)
FOR j:= 1 TO n DIV 2 DO
 FOR i:=1 TO n DIV 2 DO
  ceins.xa[i,j]:=x[i,j];
  ceins.ya[i,j]:=y[i,j];
  czwei.xa[i,j]:=x[i,j];
  cdrei.ya[i,j]:=y[i,j];
 END;
END;

(*X,Y [2,1]*)
FOR j:= 1 TO n DIV 2 DO
 FOR i:= n DIV 2 +1 TO n DO
  ceins.yb[i,j]:=y[i,j];
  cdrei.xa[i,j]:=x[i,j];
  cdrei.yb[i,j]:=y[i,j];
  cvier.xa[i,j]:=x[i,j];
 END;
END;

(*X,Y [1,2]*)
FOR j:= n DIV 2 +1 TO n DO
 FOR i:= 1 TO n DIV 2 DO
  ceins.xb[i,j]:=x[i,j];
  czwei.xb[i,j]:=x[i,j];
  czwei.ya[i,j]:=y[i,j];
  cvier.ya[i,j]:=y[i,j];
 END;
END;

(*X,Y [2,2]*)
FOR j:= n DIV 2 +1 TO n DO
 FOR i:=n DIV 2 +1 TO n DO
  czwei.yb[i,j]:=y[i,j];
  cdrei.xb[i,j]:=x[i,j];
  cvier.xb[i,j]:=x[i,j];
  cvier.yb[i,j]:=y[i,j];
 END;
END;

teins:= Thread.Fork(ceins);
tzwei:= Thread.Fork(czwei);
tdrei:= Thread.Fork(cdrei);
EVAL Mult(cvier);
EVAL Thread.Join(teins);
EVAL Thread.Join(tzwei);
EVAL Thread.Join(tdrei);

(*Z [1,1]*)
FOR j:= 1 TO n DIV 2 DO
 FOR i:=1 TO n DIV 2 DO
  c[i,j]:=ceins.zb[i,j];
 END;
END;

(*Z [2,1]*)
FOR j:= 1 TO n DIV 2 DO
 FOR i:=n DIV 2+1 TO n DO
  c[i,j]:=cdrei.zb[i,j];
 END;
END;

(*Z [1,2]*)
FOR j:= n DIV 2 +1 TO n DO
 FOR i:=1 TO n DIV 2 DO
  c[i,j]:=czwei.zb[i,j];
 END;
END;

(*Z [2,2]*)
FOR j:= n DIV 2+1 TO n DO
 FOR i:=n DIV 2+1 TO n DO
  c[i,j]:=cvier.zb[i,j];
 END;
END;

END MultPar;

(*-------------------------------------------------------------------*)
(* Prozedur Mult, liest Matrizen aus Closure, berechnet MatProd aus  *)
(* Za:= Xa*Ya und Zb:= Xb*Yb, sowie MatAdd aus Z:= Za+Zb .           *)
(*-------------------------------------------------------------------*)

PROCEDURE Mult(close : Closure) : REFANY =

BEGIN

 FOR i:=1 TO n DO

  FOR j:=1 TO n DO

    FOR k:=1 TO n DO

      IF k=1 THEN close.za[i,j]:=0 END;

      INC(close.za[i,j],close.xa[i,k]*close.ya[k,j]);

    END;

  END;

 END;

 FOR i:=1 TO n DO

  FOR j:=1 TO n DO

    FOR k:=1 TO n DO

      IF k=1 THEN close.zb[i,j]:=0 END;

      INC(close.zb[i,j],close.xb[i,k]*close.yb[k,j]);

    END;

  END;

 END;

Add(close.za,close.zb,close.zb);

RETURN NIL

END Mult;

BEGIN

END Matrix.
