MODULE BinaryTree;
IMPORT IO;

VAR newElement : REF ELEMENT;

(*-------------------------------------------------------------------*)
(* Prozedur insert, liest Argumente: Baum in den eingefuegt wird,    *)
(* (tree) einzufuegendes Element (elem), keine Rueckgabewerte.       *)
(* Fuegt Element (elem) in den Baum ein.                             *)
(*-------------------------------------------------------------------*)

PROCEDURE insert(VAR tree : RefT; elem : INTEGER)
RAISES {WrongData} =

BEGIN

  (* sucht (rekursiv) passende Position und fuegt, wenn gefunden *)
  (* als neues Element in den Baum ein                           *)
  IF tree= NIL THEN  (*Baum leer, dann direkt einfuegen*)
    newElement:= NEW(RefT);
    newElement^.key := elem;
    tree:= newElement;
  (*wenn bereits vorhanden, Fehler*)
  ELSIF tree^.key = elem THEN RAISE WrongData
  
  (*Suche passende Stelle*)
  ELSIF tree^.left#NIL AND tree^.key>elem THEN
    insert(tree^.left, elem);
  ELSIF tree^.right#NIL AND tree^.key<elem THEN
    insert(tree^.right, elem);
  
  (*Untergeordnete Elemente anhaengen und neues Element*)
  (*an uebergeordnetes Element anhaengen               *)
  ELSIF tree^.left=NIL AND tree^.key>elem THEN
    newElement:= NEW(RefT);
    newElement^.parent := tree;
    newElement^.key := elem;
    tree^.left:= newElement;
  ELSIF tree^.right=NIL AND tree^.key<elem THEN
    newElement:= NEW(RefT);
    newElement^.parent := tree;
    newElement^.key := elem;
    tree^.right:= newElement;
 
  ELSE
    RAISE WrongData;
  END;

END insert;

(*-------------------------------------------------------------------*)
(* Prozedur delete, liest Argumente: Baum aus dem entfernt wird,     *)
(* (tree) Zeiger auf zu entfernendes Element (node), keine           *)
(* Rueckgabewerte. Entfernt Element aus Baum .                       *)
(*  Aufrufe:                                                         *)
(* delete -> empty                                                   *)
(* delete -> minimum                                                 *)
(*-------------------------------------------------------------------*)

PROCEDURE delete(VAR tree : RefT; node : RefT)
RAISES {Empty} =

BEGIN

  IF empty(tree) THEN (* Fehler, wenn leer. *)
    RAISE Empty;
  END;

  (* Haengt linken Ast am Minium des rechten Asts an *)
  IF node^.right#NIL AND node^.left#NIL THEN
    newElement := minimum(node^.right);
    newElement^.left := node^.left;
  ELSIF node^.right=NIL AND node^.left#NIL THEN
    node^.right := node^.left;
  ELSE
  END;

  (* Haengt Rest an uebergeordnetem Element an *)
  (* (zu loeschendes Element wird entfernt)    *)
  IF node^.parent=NIL THEN
    node^.right^.parent:= NIL;
    tree:= node^.right;
  ELSIF node^.parent^.right= node THEN
    IF node^.right#NIL THEN
      node^.right^.parent := node^.parent;
      node^.parent^.right:= node^.right;
    ELSE node^.parent^.right:= NIL;
    END;
  ELSE
    IF node^.right#NIL THEN
      node^.right^.parent := node^.parent;
      node^.parent^.left:= node^.right;
     ELSE node^.parent^.left:= NIL;
     END;
  END;

END delete;

(*-------------------------------------------------------------------*)
(* Prozedur search, liest Argumente: Baum in dem gesucht wird,       *)
(* (tree) zu suchendes Element (key), gibt Zeiger auf gefundenes     *)
(* Element zurueck.                                                  *)
(*  Aufrufe:                                                         *)
(* search -> empty                                                   *)
(*-------------------------------------------------------------------*)

PROCEDURE search(tree : RefT; key : INTEGER) : RefT
RAISES {Empty, WrongData} =

BEGIN

  IF empty(tree) THEN (* Fehler, wenn leer. *)
   RAISE Empty;
  END;

  (* Verzweigung: gefunden, gib Zeiger zurueck, rechter < , dann *)
  (* setze fort mit rechtem Teil, linker > , dann setze fort mit *)
  (* linkem Teil, sonst Fehler.                                  *)
  WHILE tree#NIL DO
    IF tree^.key=key THEN RETURN tree
    ELSIF tree^.right#NIL AND tree^.key<key THEN
      tree:= tree^.right;
    ELSIF tree^.left#NIL AND tree^.key>key THEN
      tree:= tree^.left;
    ELSE
      RAISE WrongData;
    END;
  END;

  RAISE WrongData;

END search;

(*-------------------------------------------------------------------*)
(* Prozedur minimum, liest Argumente: Baum in dem gesucht wird,      *)
(* gibt Zeiger auf gefundes Element zureck. Geht so weit nach links *)
(* wie moeglich, gibt Zeiger auf dieses Element zurueck.             *)
(*  Aufrufe:                                                         *)
(* minimum -> empty                                                  *)
(*-------------------------------------------------------------------*)

PROCEDURE minimum(tree : RefT) : RefT
RAISES {Empty} =

BEGIN

  IF empty(tree) THEN (* Fehler, wenn leer. *)
    RAISE Empty;
  END;
  
  (*Rufe solange mit lower part auf bis kl. Element*)
  IF tree^.left#NIL THEN
    tree:= minimum(tree^.left)
  END;

  RETURN tree

END minimum;

(*-------------------------------------------------------------------*)
(* Prozedur maximum, liest Argumente: Baum in dem gesucht wird,      *)
(* gibt Zeiger auf gefundes Element zureck. Geht so weit nach rechts*)
(* wie moeglich, gibt Zeiger auf dieses Element zurueck.             *)
(*  Aufrufe:                                                         *)
(* maximum -> empty                                                  *)
(*-------------------------------------------------------------------*)

PROCEDURE maximum(tree : RefT) : RefT
RAISES {Empty} =

BEGIN

  IF empty(tree) THEN (* Fehler, wenn leer. *)
    RAISE Empty;
  END;

  (*Rufe solange mit upper part auf bis gr. Element*)
  IF tree^.right#NIL THEN
    tree:= maximum(tree^.right)
  END;

  RETURN tree

END maximum;

(*-------------------------------------------------------------------*)
(* Prozedur put, liest Argumente: Baum der ausgegeben wird,          *)
(* keine Rueckgabewerte. Gibt von links unten ausgehend nach rechts  *)
(* unten alle Elemente des Baums entlang der Aeste aus.              *)
(*  Aufrufe:                                                         *)
(* put -> empty                                                      *)
(*-------------------------------------------------------------------*)

PROCEDURE put(tree : RefT)
RAISES {Empty}  =

BEGIN

  IF empty(tree) THEN (* Fehler, wenn leer. *)
   RAISE Empty;
  END;

  IF tree^.left#NIL THEN  (* Lower Branch ausgeben *)
    put(tree^.left);
    IO.PutInt(tree^.key);
    IO.Put("\t");
  END;

  IF tree=minimum(tree) THEN
    IO.PutInt(tree^.key);
    IO.Put("\t");
  END;


  IF tree^.right#NIL THEN  (* Upper Branch ausgeben *)
    put(tree^.right);
  END;

END put;

(*-------------------------------------------------------------------*)
(* Prozedur empty, liest Argumente: Baum der geprueft wird,          *)
(* gibt wahr zurueck falls Baum leer, falsch falls Baum nicht leer   *)
(* ist.                                                              *)
(*-------------------------------------------------------------------*)

PROCEDURE empty(tree : RefT) : BOOLEAN =

BEGIN

  IF tree= NIL THEN RETURN TRUE
  ELSE RETURN FALSE
  END;

END empty;

BEGIN

END BinaryTree.

